/* dynl-beos.c - dynamic linking with BeOS "add-ons"
 *
 * Copyright (C) 1998 Ingejors firman Peter Elg
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 * As a special exception, the Free Software Foundation gives permission
 * for additional uses of the text contained in its release of GUILE.
 *
 * The exception is that, if you link the GUILE library with other files
 * to produce an executable, this does not by itself cause the
 * resulting executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of
 * linking the GUILE library code into it.
 *
 * This exception does not however invalidate any other reasons why
 * the executable file might be covered by the GNU General Public License.
 *
 * This exception applies only to the code released by the
 * Free Software Foundation under the name GUILE.  If you copy
 * code from other Free Software Foundation releases into a copy of
 * GUILE, as the General Public License permits, the exception does
 * not apply to the code that you add in this way.  To avoid misleading
 * anyone as to the status of such modified files, you must delete
 * this exception notice from them.
 *
 * If you write modifications of your own for GUILE, it is your choice
 * whether to permit this exception to apply to your modifications.
 * If you do not wish that, delete this exception notice.  */

#include <kernel/image.h>

static void *
sysdep_dynl_link (fname, subr)
     char *fname;
     char *subr;
{
    image_id image;

//	fprintf(stderr, "Trying to load add on '%s'\n", fname);
	    
	image = load_add_on (fname);

    if (image < 0)
      {
		SCM_ALLOW_INTS;
		scm_misc_error (subr, "failed to load add-on \"%s\"", 
			scm_listify (scm_makfrom0str (fname), SCM_UNDEFINED) );
      }
    return (void *)image;
}

static void
sysdep_dynl_unlink (handle, subr)
     void *handle;
     char *subr;
{
	if (unload_add_on ((image_id)handle) != B_NO_ERROR)
	  {
		SCM_ALLOW_INTS;
		scm_misc_error (subr, "failed to unload add-on (invalid image?)", SCM_EOL);
      }
}

static void *
sysdep_dynl_func (symb, handle, subr)
     char *symb;
     void *handle;
     char *subr;
{
    void *func;

	if (get_image_symbol ((image_id)handle, symb, B_SYMBOL_TYPE_TEXT, &func)
		!= B_OK) {
		SCM_ALLOW_INTS;
		scm_misc_error (subr, "failed to load symbol \"%s\"", 
				scm_listify (scm_makfrom0str (symb), SCM_UNDEFINED) );
	}
    return func;
}

static void
sysdep_dynl_init ()
{
}
